# (C) Copyright 2016 Hewlett Packard Enterprise Development LP

import logging

import monasca_setup.agent_config
import monasca_setup.detection

log = logging.getLogger(__name__)

CA_CERTS = "/etc/ssl/certs/ca-certificates.crt"
CIPHERS = "HIGH:-aNULL:-eNULL:-PSK:RC4-SHA:RC4-MD5"
DEFAULT_TIMEOUT = 1.0
DEFAULT_COLLECT_PERIOD = 3600


class CertificateCheck(monasca_setup.detection.ArgsPlugin):
    """Setup a https certification check according to the passed in args.

       Outputs one metric: https.cert_expire_days which is the number of
       days until the certificate expires

       Despite being a detection plugin, this plugin does no detection and
       will be a NOOP without arguments.  Expects one argument, 'urls'
       which is a comma-separated list of urls
       Examples:

       monasca-setup -d CertificateCheck -a "urls=https://ThisCloud.example:8070"
       These arguments are optional:
       ca_certs: file containing the certificates for Certificate Authorities
                 default is CA_CERTS
       ciphers: list of ciphers to check
                default is CIPHERS
       collect_period: Integer time in seconds between outputting the metric.
                       Since the metric is in days, it makes sense to output
                       it at a slower rate. The default is once per hour
       timeout: Float time in seconds before timing out the connect to the url.
                Increase if needed for very slow servers, but making this too
                long will increase the time this plugin takes to run if the
                server for the url is down. The default is 1.0 seconds
    """

    def _detect(self):
        """Run detection, set self.available True if urls are detected
        """
        self.available = self._check_required_args(['urls'])

    def build_config(self):
        """Build the config as a Plugins object and return.
        """
        config = monasca_setup.agent_config.Plugins()
        instances = []
        init_config = {'ca_certs': CA_CERTS,
                       'ciphers': CIPHERS,
                       'timeout': DEFAULT_TIMEOUT,
                       'collect_period': DEFAULT_COLLECT_PERIOD}
        if 'ca_certs' in self.args:
            init_config['ca_certs'] = self.args['ca_certs']
        if 'ciphers' in self.args:
            init_config['ciphers'] = self.args['ciphers']
        if 'timeout' in self.args:
            timeout = float(self.args['timeout'])
            if timeout <= 0.0:
                log.error('Invalid timeout value %s, ignoring' %
                          self.args['timeout'])
            else:
                init_config['timeout'] = timeout
        if 'collect_period' in self.args:
            collect_period = int(self.args['collect_period'])
            init_config['collect_period'] = collect_period
        for url in self.args['urls'].split(','):
            url = url.strip()
            # Allow comma terminated lists
            if not url:
                continue
            log.info("\tAdding SSL Certificate expiration check for {}".format(url))
            instance = self._build_instance([])
            instance.update({'url': url, 'name': url})
            instances.append(instance)

        config['cert_check'] = {'init_config': init_config,
                                'instances': instances}

        return config



# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP

import logging
import os
import pkg_resources
import six
import yaml

from monasca_agent.common.exceptions import PathNotFound
import monasca_agent.common.singleton as singleton

DEFAULT_CONFIG_FILE = '/etc/monasca/agent/agent.yaml'
DEFAULT_LOG_DIR = '/var/log/monasca/agent'
LOGGING_MAX_BYTES = 5 * 1024 * 1024

log = logging.getLogger(__name__)


# Make this a singleton class so we don't get the config every time
# the class is created
@six.add_metaclass(singleton.Singleton)
class Config(object):

    def __init__(self, configFile=None):
        # importing it here, in order to avoid a circular import
        # as monasca_agent.common.util imports this module.
        from monasca_agent.common import util

        options, args = util.get_parsed_args()
        if configFile is not None:
            self._configFile = configFile
        elif options.config_file is not None:
            self._configFile = options.config_file
        elif os.path.exists(DEFAULT_CONFIG_FILE):
            self._configFile = DEFAULT_CONFIG_FILE
        elif os.path.exists(os.getcwd() + '/agent.yaml'):
            self._configFile = os.getcwd() + '/agent.yaml'
        else:
            error_msg = 'No config file found at {0} nor in the working directory.'.format(DEFAULT_CONFIG_FILE)
            log.error(error_msg)
            raise IOError(error_msg)

        # Define default values for the possible config items
        self._config = {'Main': {'check_freq': 15,
                                 'forwarder_url': 'http://localhost:17123',
                                 'hostname': None,
                                 'dimensions': None,
                                 'listen_port': None,
                                 'version': self.get_version(),
                                 'additional_checksd': '/usr/lib/monasca/agent/custom_checks.d',
                                 'limit_memory_consumption': None,
                                 'skip_ssl_validation': False,
                                 'autorestart': True,
                                 'non_local_traffic': False,
                                 'sub_collection_warn': 6,
                                 'collector_restart_interval': 24},
                        'Api': {'is_enabled': False,
                                'url': '',
                                'project_name': '',
                                'project_id': '',
                                'project_domain_name': '',
                                'project_domain_id': '',
                                'ca_file': '',
                                'insecure': '',
                                'username': '',
                                'password': '',
                                'use_keystone': True,
                                'keystone_timeout': 20,
                                'keystone_url': '',
                                'max_buffer_size': 1000,
                                'max_measurement_buffer_size': -1,
                                'write_timeout': 10,
                                'backlog_send_rate': 5},
                        'Statsd': {'recent_point_threshold': None,
                                   'monasca_statsd_interval': 20,
                                   'monasca_statsd_forward_host': None,
                                   'monasca_statsd_forward_port': 8125,
                                   'monasca_statsd_port': 8125},
                        'Logging': {'disable_file_logging': False,
                                    'log_level': None,
                                    'collector_log_file': DEFAULT_LOG_DIR + '/collector.log',
                                    'forwarder_log_file': DEFAULT_LOG_DIR + '/forwarder.log',
                                    'statsd_log_file': DEFAULT_LOG_DIR + '/statsd.log',
                                    'jmxfetch_log_file': DEFAULT_LOG_DIR + '/jmxfetch.log',
                                    'log_to_event_viewer': False,
                                    'log_to_syslog': False,
                                    'syslog_host': None,
                                    'syslog_port': None}}

        self._read_config()

    def get_config(self, sections='Main'):
        """Get the config info."""
        section_list = []
        if isinstance(sections, six.string_types):
            section_list.append(sections)
        elif isinstance(sections, list):
            section_list.extend(sections)
        else:
            log.error('Unknown section: {0}'.format(str(sections)))
            return {}

        new_config = {}
        for section in section_list:
            new_config.update(self._config[section])

        return new_config

    def get_version(self):
        return pkg_resources.require("monasca-agent")[0].version

    def _read_config(self):
        """Read in the config file."""
        try:
            with open(self._configFile, 'r') as f:
                log.debug('Loading config file from {0}'.format(self._configFile))
                config = yaml.safe_load(f.read())
                [self._config[section].update(config[section]) for section in config.keys()]
        except Exception as e:
            log.exception('Error loading config file from {0}'.format(self._configFile))
            raise e

    def get_confd_path(self):
        path = os.path.join(os.path.dirname(self._configFile), 'conf.d')
        if os.path.exists(path):
            return path
        raise PathNotFound(path)

    def check_yaml(self, conf_path):
        f = open(conf_path)
        try:
            check_config = yaml.safe_load(f.read())
            assert 'init_config' in check_config, "No 'init_config' section found"
            assert 'instances' in check_config, "No 'instances' section found"

            valid_instances = True
            if check_config['instances'] is None or not isinstance(check_config['instances'], list):
                valid_instances = False
            else:
                for i in check_config['instances']:
                    if not isinstance(i, dict):
                        valid_instances = False
                        break
            if not valid_instances:
                raise Exception(
                    'You need to have at least one instance defined in the YAML file for this check')
            else:
                return check_config
        finally:
            f.close()


def main():
    configuration = Config()
    config = configuration.get_config()
    api_config = configuration.get_config('Api')
    statsd_config = configuration.get_config('Statsd')
    logging_config = configuration.get_config('Logging')
    print("Main Configuration: \n {0}".format(config))
    print("\nApi Configuration: \n {0}".format(api_config))
    print("\nStatsd Configuration: \n {0}".format(statsd_config))
    print("\nLogging Configuration: \n {0}".format(logging_config))


if __name__ == "__main__":
    logging.basicConfig()
    main()



# (C) Copyright 2016 Hewlett Packard Enterprise Development LP
"""VCenter Only.

Generic VCenter check. This check allows you to specify particular metrics that
you want from vCenter in your configuration.
"""

import json
import logging as log
from monasca_agent.collector.checks import AgentCheck
from monasca_agent.common.config import Config
from oslo_vmware import api
from oslo_vmware import vim_util
import requests
import traceback

CLUSTER_COMPUTE_PROPERTIES = ["name", "host", "datastore"]
CLUSTER_HOST_PROPERTIES = ['runtime.connectionState',
                           'runtime.inMaintenanceMode',
                           'summary.hardware.numCpuThreads',
                           'summary.hardware.memorySize',
                           'summary.hardware.cpuMhz']
CLUSTER_PERF_COUNTERS = [
    "cpu.usagemhz.average",
    "cpu.totalmhz.average",
    "mem.usage.average",
    "mem.consumed.average",
    "mem.totalmb.average"]
STORAGE_VOLUME_PROPERTIES = ["host",
                             "summary.capacity",
                             "summary.freeSpace",
                             "summary.name",
                             "summary.multipleHostAccess",
                             "summary.accessible",
                             "summary.maintenanceMode",
                             "summary.type"]

CPU_TOTAL_MHZ = "vcenter.cpu.total_mhz"
CPU_USED_MHZ = "vcenter.cpu.used_mhz"
CPU_USED_PERCENT = "vcenter.cpu.used_perc"
CPU_TOTAL_LOGICAL_CORES = "vcenter.cpu.total_logical_cores"
MEMORY_TOTAL_MB = "vcenter.mem.total_mb"
MEMORY_USED_MB = "vcenter.mem.used_mb"
MEMORY_USED_PERCENT = "vcenter.mem.used_perc"
DISK_TOTAL_SPACE_MB = "vcenter.disk.total_space_mb"
DISK_TOTAL_USED_SPACE_MB = "vcenter.disk.total_used_space_mb"
DISK_TOTAL_USED_SPACE_PERCENT = "vcenter.disk.total_used_space_perc"

PERF_MANAGER_TYPE = "PerformanceManager"
PERF_RESOURCE = 'ClusterResource'
NUM_SAMPLES = 15
SAMPLE_RATE = 300
ALLOWED_DATASTORE_TYPES = ['VMFS', 'NFS']


class VCenterCheck(AgentCheck):

    def __init__(self, name, init_config, agent_config):
        AgentCheck.__init__(self, name, init_config, agent_config)
        self._max_objects = 1000
        self.session = None
        self.is_new_session = True
        self._resource_moid_dict = {}

    def _propset_dict(self, propset):
        """Turns a propset list into a dictionary

        PropSet is an optional attribute on ObjectContent objects
        that are returned by the VMware API.
        :param propset: a property 'set' from ObjectContent
        :return: dictionary representing property set
        """
        if propset is None:
            return {}
        return {prop.name: prop.val for prop in propset}

    def _get_api_session(self):
        api_session = api.VMwareAPISession(
            self.vcenter_ip,
            self.user,
            self.password,
            3,  # retry count
            0.5,  # task_poll_interval
            port=self.port,
            scheme="https")
        return api_session

    def _find_entity_mor(self, entity_list, entity_name):
        for object_contents in entity_list:
            for obj_content in object_contents[1]:
                for dyn_props in obj_content.propSet:
                    if dyn_props.val == entity_name.decode('utf-8'):
                        return obj_content

    def _get_resource_dict_by_name(self, resource_name):
        """Return moid from the specified resource"""
        self._get_cluster_info_dict()
        self.log.debug("Loaded MORs for clusters %s" %
                       str(self._resource_moid_dict.keys()))
        return self._resource_moid_dict.get(resource_name)

    def _build_resource_dict(self, object_content):
        """Method to build the resource dictionary. Cluster_name is the key,
        and its host and datastore names are the values
        {'cluster_name': {
            'moid': 'cluster1',
            'host': ['h1', 'h2'],
            'datastore': ['ds1', 'ds2']
            }
        }
        """
        cluster_moid = str(object_content.obj.value)
        # extract host and datastore names
        host_names = []
        datastore_names = []
        dict_key = None
        propSet = object_content.propSet
        for dynamic_prop in propSet:
            mor_array = dynamic_prop.val
            if dynamic_prop.name == 'name':
                dict_key = dynamic_prop.val
            else:
                for obj in mor_array:
                    for mor in obj[1]:
                        if dynamic_prop.name == 'datastore':
                            datastore_names.append(str(mor.value))
                        elif dynamic_prop.name == 'host':
                            host_names.append(str(mor.value))
        if dict_key:
            self._resource_moid_dict[str(dict_key)] = {
                'moid': cluster_moid,
                'host': host_names,
                'datastore': datastore_names,
            }

    def _get_cluster_info_dict(self):
        """Load configured cluster moids from the vCenter"""
        self.log.debug("Inside _get_cluster_info_dict")
        if self.session is None:
            self.session = self._get_api_session()
        result = self.session.invoke_api(vim_util,
                                         "get_objects",
                                         self.session.vim,
                                         "ClusterComputeResource",
                                         self._max_objects,
                                         CLUSTER_COMPUTE_PROPERTIES)
        for name in self.clusters:
            mor = self._find_entity_mor(result, name)
            if mor:
                self._build_resource_dict(mor)

    def _get_sample(self, samples, counter_name, is_summation=False):
            res = 0
            num_samples = 0

            for cn in samples:
                if cn.startswith(counter_name):
                    vals = samples[cn]
                    if vals:
                        for val in vals:
                            i_val = int(val)
                            if i_val != -1:
                                res += i_val
                                num_samples += 1

            if not is_summation and num_samples:
                res /= num_samples

            return res

    def _get_shared_datastores(self, datastore_stats, managed_cluster):
        """Method to find the shared datastores associated with the cluster
        Also handles cluster having single host with a non-shared datastore.
        """
        shared_datastores_ids = []
        res_dict = self._get_resource_dict_by_name(managed_cluster)
        host_set = set(res_dict.get('host'))
        for object_contents in datastore_stats:
            for object_content in object_contents[1]:
                ds_mor = object_content.obj.value
                datastore_host_ids = []
                if ds_mor in res_dict.get('datastore'):
                    propSets = object_content.propSet
                    for propSet in propSets:
                        if propSet.name == 'host':
                            ds_hm_array = propSet.val
                            host_mounts = ds_hm_array[0]
                            for host_mount in host_mounts:
                                datastore_host_ids.append(host_mount.key.value)
                    if host_set.issubset(set(datastore_host_ids)):
                        shared_datastores_ids.append(ds_mor)
                    self.log.debug("Cluster host list ==" + str(host_set))
                    self.log.debug("Datastore host list ==" +
                                   str(datastore_host_ids))
        self.log.debug("shared_datastores_ids==" +
                       str(shared_datastores_ids))
        return shared_datastores_ids

    def _is_valid_datastore(self, propSets):
        propdict = self._propset_dict(propSets)
        return (propdict.get('summary.accessible') and
                (propdict.get('summary.maintenanceMode') is None or
                 propdict.get('summary.maintenanceMode') == 'normal') and
                propdict['summary.type'] in ALLOWED_DATASTORE_TYPES)

    def _process_storage_data(self, datastore_stats, managed_cluster):
        shared_ds = self._get_shared_datastores(datastore_stats,
                                                managed_cluster)
        capacity = 0L
        freeSpace = 0L
        self.log.info("Polling for the datastores: " + str(shared_ds))
        for object_contents in datastore_stats:
            for object_content in object_contents[1]:
                ds_mor = object_content.obj.value
                if ds_mor in shared_ds:
                    propSets = object_content.propSet
                    if self._is_valid_datastore(propSets):
                        for propSet in propSets:
                            if propSet.name == 'summary.capacity':
                                self.log.debug("Calculating capacity "
                                               "of datastore: %s in cluster: "
                                               "%s" %
                                               (ds_mor, managed_cluster))
                                capacity += long(propSet.val)
                            elif propSet.name == 'summary.freeSpace':
                                self.log.debug("Calculating freeSpace of "
                                               "datastore: %s in cluster: %s"
                                               % (ds_mor, managed_cluster))
                                freeSpace += long(propSet.val)
        usedSpace = capacity - freeSpace
        self.log.debug("Total capacity:" + str(capacity) +
                       " used:" + str(usedSpace) + " free:" + str(freeSpace))
        return (capacity, usedSpace)

    def _get_properties_for_a_collection_of_objects(self, vim, type, obj_list,
                                                    properties):
        """Gets the list of properties for the collection of
        objects of the type specified.
        """
        client_factory = vim.client.factory
        if len(obj_list) == 0:
            return []
        prop_spec = vim_util.build_property_spec(client_factory,
                                                 type,
                                                 properties)
        lst_obj_specs = []
        for obj in obj_list:
            lst_obj_specs.append(vim_util.build_object_spec(
                client_factory,
                obj, []))
        prop_filter_spec = vim_util.build_property_filter_spec(client_factory,
                                                               [prop_spec],
                                                               lst_obj_specs)
        options = client_factory.create('ns0:RetrieveOptions')
        options.maxObjects = self._max_objects
        return vim.RetrievePropertiesEx(
            vim.service_content.propertyCollector,
            specSet=[prop_filter_spec],
            options=options)

    def _is_valid_host(self, propSets):
        propdict = self._propset_dict(propSets)
        return ((propdict.get('runtime.inMaintenanceMode') is not True) and
                propdict.get('runtime.connectionState') == 'connected')

    def _process_host_data(self, host_stats, managed_cluster):
        shared_hosts = self._find_entity_mor(host_stats, managed_cluster)
        cluster_hosts_data = {'numCpuThreads': 0,
                              'memorySizeMb': 0,
                              'cpuMhz': 0}
        self.log.debug("Polling for the hosts: %s" % (str(shared_hosts)))
        if shared_hosts:
            propSets = shared_hosts.propSet
            for propSet in propSets:
                if propSet.name == 'host':
                    host_ret = propSet.val
                    host_mors = host_ret.ManagedObjectReference
        if host_mors:
            method_call = '_get_properties_for_a_collection_of_objects'
            result = self.session.invoke_api(self,
                                             method_call,
                                             self.session.vim,
                                             "HostSystem",
                                             host_mors,
                                             CLUSTER_HOST_PROPERTIES)
        for obj_contents in result:
            for obj_content in obj_contents[1]:
                propSets = obj_content.propSet
                if self._is_valid_host(propSets):
                    for propSet in propSets:
                        if propSet.name == "summary.hardware.numCpuThreads":
                            cluster_hosts_data['numCpuThreads'] += propSet.val
                        elif propSet.name == 'summary.hardware.memorySize':
                            cluster_hosts_data['memorySizeMb'] += int(
                                (propSet.val) / (1024 * 1024))
                        elif propSet.name == 'summary.hardware.cpuMhz':
                            cluster_hosts_data['cpuMhz'] += propSet.val
        return cluster_hosts_data

    def check(self, instance):
        try:
            if self.is_new_session:
                self.instance = instance
                self.vcenter_ip = instance.get('vcenter_ip', None)
                self.user = instance.get('username', None)
                self.password = instance.get('password', None)
                self.port = instance.get('port', 443)
                self.clusters = instance.get('clusters', None)
                if not self.vcenter_ip:
                    self.log.warn("vCenter not configured")
                    return
                if not self.clusters:
                    self.log.warn("No clusters configured to monitor")
                    return
                self.session = self._get_api_session()
                self.vc_uuid = (self.session.vim.service_content.
                                about.instanceUuid)
                self._ops = VcenterOperations(self.session,
                                              self._max_objects,
                                              self.log)
                self._ops._properties_updated_event()
                self.perf_query_specs = self._ops._get_perf_query_spec(
                    CLUSTER_PERF_COUNTERS,
                    SAMPLE_RATE,
                    NUM_SAMPLES)
                self.is_new_session = False
            for managed_cluster in self.clusters:
                try:
                    res_dict = self._get_resource_dict_by_name(managed_cluster)
                    if not res_dict:
                        self.log.error("Configured cluster not found "
                                       "in vCenter")
                        continue
                    self.log.debug("=" * 30 + " Start of cycle" + "=" * 30)
                    self.log.debug("Collecting for cluster: %s (%s)" %
                                   (managed_cluster,
                                    str(res_dict.get('moid'))))
                    cluster_samples = self._ops.\
                        _get_perf_stats(res_dict,
                                        self.perf_query_specs)
                    if not cluster_samples:
                        self.log.error("Activated cluster not found in"
                                       " vCenter")
                        continue

                    host_stats = self._ops._get_host_stats_from_cluster()
                    cluster_host_stats = self._process_host_data(
                        host_stats,
                        managed_cluster)
                    cpu_total_logical_cores = cluster_host_stats.get(
                        'numCpuThreads')

                    cpu_usage = self._get_sample(cluster_samples,
                                                 'cpu.usagemhz.average')
                    cpu_total = self._get_sample(cluster_samples,
                                                 'cpu.totalmhz.average')
                    cpu_usage_percent = float(0.0)
                    if cpu_total > 0:
                        cpu_usage_percent = round(
                            (float(cpu_usage) / cpu_total) * 100, 2)

                    # Considering host level stats for RAM
                    mem_mb_total = cluster_host_stats.get('memorySizeMb')
                    mem_kb_consumed = self._get_sample(cluster_samples,
                                                       'mem.consumed.average')
                    # convert this to MB
                    mem_mb_consumed = mem_kb_consumed / 1024
                    mem_used_percent = float(0.0)
                    if mem_mb_total > 0:
                        mem_used_percent = round(
                            (float(mem_mb_consumed) / mem_mb_total) * 100, 2)

                    datastore_stats = self._ops._read_storage_data()
                    (capacity_bytes, usedSpace_bytes) = \
                        self._process_storage_data(
                            datastore_stats,
                            managed_cluster)
                    storage_used_percent = float(0.0)
                    if capacity_bytes > 0:
                        storage_used_percent = round(
                            float(usedSpace_bytes) / capacity_bytes, 2) * 100
                    capacity_mb = capacity_bytes / (1024 * 1024)
                    usedSpace_mb = usedSpace_bytes / (1024 * 1024)

                    self.log.debug(managed_cluster +
                                   ", CPU used (avg MHz): %s" % str(cpu_usage))
                    self.log.debug(managed_cluster +
                                   ", CPU total (avg MHz): %s" %
                                   str(cpu_total))
                    self.log.debug(managed_cluster +
                                   ", Memory total (MB): %s" %
                                   str(mem_mb_total))
                    self.log.debug(managed_cluster +
                                   ", Memory consumed (MB): %s" %
                                   str(mem_mb_consumed))
                    self.log.debug(managed_cluster +
                                   ", Storage total (MB): %s" %
                                   str(capacity_mb))
                    self.log.debug(managed_cluster +
                                   ", Storage used (MB): %s" %
                                   str(usedSpace_mb))
                    self.log.debug(managed_cluster +
                                   ", CPU cores : %s" %
                                   str(cpu_total_logical_cores))
                    self.log.debug(managed_cluster +
                                   ", CPU cores: %s" %
                                   str(cpu_total_logical_cores) +
                                   ", CPU usage percent: %s" %
                                   str(cpu_usage_percent) +
                                   ", Memory usage percent: %s" %
                                   str(mem_used_percent) +
                                   ", Storage usage percent: %s" %
                                   str(storage_used_percent))

                    data = {
                        CPU_TOTAL_MHZ: cpu_total,
                        CPU_USED_MHZ: cpu_usage,
                        CPU_USED_PERCENT: cpu_usage_percent,
                        CPU_TOTAL_LOGICAL_CORES: cpu_total_logical_cores,
                        MEMORY_TOTAL_MB: mem_mb_total,
                        MEMORY_USED_MB: mem_mb_consumed,
                        MEMORY_USED_PERCENT: mem_used_percent,
                        DISK_TOTAL_SPACE_MB: capacity_mb,
                        DISK_TOTAL_USED_SPACE_MB: usedSpace_mb,
                        DISK_TOTAL_USED_SPACE_PERCENT: storage_used_percent
                    }

                    self._set_metrics(managed_cluster,
                                      data,
                                      res_dict.get("moid"))
                    self.log.debug("=" * 30 + " End of cycle" + "=" * 30)
                except Exception:
                    self.log.error(traceback.format_exc())
                    self.log.error("Exception occurred while polling for %s"
                                   % managed_cluster)
        except Exception as e:
            self.is_new_session = False
            self.log.error("Exception: %s" % str(e))
            try:
                self.session.close()
            except Exception:
                self.log.error(traceback.format_exc())
            raise e

    def _set_metrics(self, managed_cluster, data, mor_id):
        dimensions = self._get_dims(managed_cluster, mor_id)

        for key in data.keys():
            self.gauge(key, data.get(key), dimensions=dimensions)
            self.log.debug("Post metric data for %s,  %s: %d" %
                           (managed_cluster, key, data.get(key)))

    def _get_dims(self, cluster, mor_id):
        cluster_id = mor_id + "." + self.vc_uuid
        local_dimensions = {
            "vcenter_ip": self.vcenter_ip,
            "esx_cluster_id": cluster_id
        }
        final_dimensions = self._set_dimensions(local_dimensions,
                                                self.instance)
        return final_dimensions

    @staticmethod
    def parse_agent_config(agentConfig):
        if not agentConfig.get('vcenter_ip'):
            return False

        vcenter_ip = agentConfig.get('vcenter_ip')
        user = agentConfig.get('username')
        paswd = agentConfig.get('password')
        config = {
            'instances': [{
                'vcenter_ip': vcenter_ip,
                'username': user,
                'password': paswd,
            }]
        }
        return config


class VcenterOperations(object):
    """Class to invoke vCenter APIs calls.

    vCenter APIs calls are required by various pollsters, collecting data from
    VMware infrastructure.
    """
    def __init__(self, session, max_objects, logger):
        self.session = session
        self.max_objects = max_objects
        self.log = logger
        self.counters = {}

    def _get_perf_counters(self):
        try:
            self.log.info("Loading VmWare performance counters")
            perf_counters = []
            property_dict = {'perf_counter': ['perfCounter']}

            prop_collector = self.session.vim.service_content.propertyCollector
            self.client_factory = self.session.vim.client.factory
            perf_manager = self.session.vim.service_content.perfManager
            options = self.client_factory.create('ns0:RetrieveOptions')
            options.maxObjects = 1

            prop_spec = vim_util.build_property_spec(
                self.client_factory,
                PERF_MANAGER_TYPE,
                property_dict.get('perf_counter'))
            obj_spec = vim_util.build_object_spec(
                self.client_factory,
                perf_manager,
                None)

            filter_spec = vim_util.build_property_filter_spec(
                self.client_factory, [prop_spec], [obj_spec])

            object_contents = self.session.invoke_api(
                self.session.vim,
                "RetrievePropertiesEx",
                prop_collector,
                specSet=[filter_spec],
                options=options)

            if object_contents is not None:
                for object_content in object_contents:
                    dynamic_properties = object_content[1][0].propSet
                    for dynamic_property in dynamic_properties:
                        perf_counters = dynamic_property.val.PerfCounterInfo

            return perf_counters
        except Exception as ex:
            self.log.error("Exception in _get_perf_counters: %s"
                           % str(ex.message))

    def _properties_updated_event(self):
        perfCounters = self._get_perf_counters()

        if perfCounters is None:
            return

        for perfCounter in perfCounters:
            counter_id = perfCounter.key
            counter_group = perfCounter.groupInfo.key
            counter_name = perfCounter.nameInfo.key
            counter_rollup_type = str(perfCounter.rollupType)
            full_counter_name = counter_group + "." + \
                counter_name + "." + counter_rollup_type
            if (full_counter_name in CLUSTER_PERF_COUNTERS):
                self.counters[full_counter_name] = perfCounter
                self.counters[counter_id] = full_counter_name

    def _get_perf_counter_ids(self, counter_names):
        res = []
        for counter_name in counter_names:
            perf_counter_info = self.counters.get(counter_name)
            if perf_counter_info is not None:
                res.append(perf_counter_info.key)
        return res

    def _get_perf_query_spec(self,
                             counter_names,
                             sample_rate,
                             num_samples):
        counter_ids = self._get_perf_counter_ids(counter_names)

        perf_metric_ids = []
        for cid in counter_ids:
            metric = self.client_factory.create('ns0:PerfMetricId')
            metric.counterId = cid
            metric.instance = '*'
            perf_metric_ids.append(metric)
        return perf_metric_ids

    def _get_perf_stats(self, entity, perf_metric_ids):
        perf_query_spec = self.client_factory.create('ns0:PerfQuerySpec')
        perf_query_spec.entity = entity.get('moid')
        perf_query_spec.metricId = perf_metric_ids
        perf_query_spec.intervalId = SAMPLE_RATE
        perf_query_spec.maxSample = NUM_SAMPLES
        perf_manager = self.session.vim.service_content.perfManager
        perf_entity_metric_base = self.session.invoke_api(
            self.session.vim,
            'QueryPerf',
            perf_manager,
            querySpec=[perf_query_spec])

        perf_result = {}
        if perf_entity_metric_base:
            for i in range(0, len(perf_entity_metric_base)):
                perf_entity_metric_csv = perf_entity_metric_base[i]
                perf_metric_series_csvs = []
                perf_metric_series_csvs = perf_entity_metric_csv.value

                if perf_metric_series_csvs is None or \
                        len(perf_metric_series_csvs) == 0:
                    continue

                for j in range(0, len(perf_metric_series_csvs)):
                    perf_metric_series_csv = perf_metric_series_csvs[j]
                    name = self.counters[perf_metric_series_csv.id.counterId]
                    instance = perf_metric_series_csv.id.instance
                    if (instance is not None and
                       len(instance) > 0 and
                       instance is not "*"):
                        name += "." + instance
                    perf_result[name] = perf_metric_series_csv.value

        return perf_result

    def _read_storage_data(self):
        datastore_stats = self.session.invoke_api(
            vim_util,
            "get_objects",
            self.session.vim,
            "Datastore",
            self.max_objects,
            STORAGE_VOLUME_PROPERTIES
        )
        return datastore_stats

    def _get_host_stats_from_cluster(self):
        prop_dict = self.session.invoke_api(vim_util,
                                            "get_objects",
                                            self.session.vim,
                                            "ClusterComputeResource",
                                            self.max_objects,
                                            ['host', 'name'])
        return prop_dict



import unittest
from tests.common import load_check
from nose.plugins.skip import SkipTest


class GearmanTestCase(unittest.TestCase):

    def testMetrics(self):
        raise SkipTest('Requires Gearman')

        config = {
            'instances': [{
                'dimensions': {'first': 'second'}
            }]
        }
        agent_config = {
            'version': '0.1',
            'api_key': 'toto'
        }

        self.check = load_check('gearmand', config, agent_config)

        self.check.check(config['instances'][0])

        metrics = self.check.get_metrics()
        self.assertIsInstance(metrics, list)
        self.assertTrue(len(metrics) == 4)
        self.assertTrue(len([k for k in metrics if "second" in k[3]['dimensions']['first']]) == 4)



#!/bin/env python
# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP
"""Monitoring Agent plugin for HTTP/API checks.

"""

import json
from numbers import Number

import monasca_agent.collector.checks.services_checks as services_checks
import monasca_agent.collector.checks_d.http_check as http_check


class HTTPMetrics(http_check.HTTPCheck):

    def __init__(self, name, init_config, agent_config, instances=None):
        super(HTTPMetrics, self).__init__(name, init_config, agent_config,
                                          instances)
        self.metric_method = {
            'gauge': self.gauge,
            'counter': self.increment,
            'rate': self.rate}

    def _valid_number(self, value, name):
        if not isinstance(value, Number):
            self.log.info("Value '{0}' is not a number for metric {1}".format(
                value, name))
            return False
        return True

    def _check(self, instance):
        addr = instance.get("url", None)
        whitelist = instance.get("whitelist", None)

        dimensions = self._set_dimensions({'url': addr}, instance)

        success, result_string = self._http_check(instance)

        if success:
            json_data = json.loads(result_string)

            for metric in whitelist:
                try:
                    metric_name = metric['name']
                    metric_type = metric['type']
                    keys = metric['path'].split('/')
                except Exception:
                    self.log.warning("Invalid configuration for metric '{0}'".format(metric))
                    continue

                current = json_data
                try:
                    for key in keys:
                        current = current[key]
                except Exception:
                    self.log.warning("Could not find a value at {0} in json message".format(keys))
                    continue

                value = current

                # everything requires a number
                if metric_type in ['gauge', 'counter', 'rate']:
                    if not self._valid_number(value, metric_name):
                        self.log.warning("Invalid value '{0}' for metric '{1}'".format(value, metric_name))
                        continue

                if metric_type in self.metric_method:
                    self.metric_method[metric_type](metric_name,
                                                    value,
                                                    dimensions=dimensions)
                else:
                    self.log.warning("Unrecognized type '{0}' for metric '{1}'".format(metric_type, metric_name))

            success_string = '{0} is UP'.format(addr)
            self.log.debug(success_string)
            return services_checks.Status.UP, success_string



# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP
""" Aggregation classes used by the collector and statsd to batch messages sent to the forwarder.
"""
import json
import logging
import re
from time import time

import monasca_agent.common.metrics as metrics_pkg


log = logging.getLogger(__name__)

# This is used to ensure that metrics with a timestamp older than
# RECENT_POINT_THRESHOLD_DEFAULT seconds (or the value passed in to
# the MetricsAggregator constructor) get discarded rather than being
# input into the incorrect bucket. Currently, the MetricsAggregator
# does not support submitting values for the past, and all values get
# submitted for the timestamp passed into the flush() function.
RECENT_POINT_THRESHOLD_DEFAULT = 3600
VALUE_META_MAX_NUMBER = 16
VALUE_META_VALUE_MAX_LENGTH = 2048
VALUE_META_NAME_MAX_LENGTH = 255

invalid_chars = "<>={}(),\"\\\\;&"
restricted_dimension_chars = re.compile('[' + invalid_chars + ']')
restricted_name_chars = re.compile('[' + invalid_chars + ' ' + ']')


class InvalidMetricName(Exception):
    pass


class InvalidDimensionKey(Exception):
    pass


class InvalidDimensionValue(Exception):
    pass


class InvalidValue(Exception):
    pass


class InvalidValueMeta(Exception):
    pass


class MetricsAggregator(object):
    """A metric aggregator class."""

    def __init__(self, hostname, recent_point_threshold=None, tenant_id=None):
        self.total_count = 0
        self.count = 0
        self.hostname = hostname
        self.global_delegated_tenant = tenant_id

        recent_point_threshold = recent_point_threshold or RECENT_POINT_THRESHOLD_DEFAULT
        self.recent_point_threshold = int(recent_point_threshold)
        self.num_discarded_old_points = 0

        self.metrics = {}

    def flush(self):
        # Flush samples.  The individual metrics reset their internal samples
        # when required
        metrics = []
        for context, metric in self.metrics.items():
            try:
                metrics.extend(metric.flush())
            except Exception:
                log.exception('Error flushing {0} metrics.'.format(metric.name))

        # Log a warning regarding metrics with old timestamps being submitted
        if self.num_discarded_old_points > 0:
            log.warn('{0} points were discarded as a result of having an old timestamp'.format(
                     self.num_discarded_old_points))
            self.num_discarded_old_points = 0

        # Save some stats.
        log.debug("received {0} payloads since last flush".format(self.count))
        self.total_count += self.count
        self.count = 0
        return metrics

    def get_hostname_to_post(self, hostname):
        if 'SUPPRESS' == hostname:
            return None
        return hostname or self.hostname

    def packets_per_second(self, interval):
        if interval == 0:
            return 0
        return round(float(self.count) / interval, 2)

    def _valid_value_meta(self, value_meta, name, dimensions):
        if len(value_meta) > VALUE_META_MAX_NUMBER:
            msg = "Too many valueMeta entries {0}, limit is {1}: {2} -> {3} valueMeta {4}"
            log.error(msg.format(len(value_meta), VALUE_META_MAX_NUMBER, name, dimensions, value_meta))
            return False
        for key, value in value_meta.iteritems():
            if not key:
                log.error("valueMeta name cannot be empty: {0} -> {1}".format(name, dimensions))
                return False
            if len(key) > VALUE_META_NAME_MAX_LENGTH:
                msg = "valueMeta name {0} must be {1} characters or less: {2} -> {3}"
                log.error(msg.format(key, VALUE_META_NAME_MAX_LENGTH, name, dimensions))
                return False

        try:
            if get_value_meta_overage(value_meta):
                msg = "valueMeta name value combinations must be {0} characters or less: {1} -> {2} valueMeta {3}"
                log.error(msg.format(VALUE_META_VALUE_MAX_LENGTH, name, dimensions, value_meta))
                return False
        except Exception:
                log.error("Unable to serialize valueMeta into JSON: {2} -> {3}".format(name, dimensions))
                return False

        return True

    def submit_metric(self, name, value, metric_class, dimensions=None,
                      delegated_tenant=None, hostname=None, device_name=None,
                      value_meta=None, timestamp=None, sample_rate=1):
        if dimensions:
            for k, v in dimensions.iteritems():
                if not isinstance(k, (str, unicode)):
                    log.error("invalid dimension key {0} must be a string: {1} -> {2}".format(k, name, dimensions))
                    raise InvalidDimensionKey
                if len(k) > 255 or len(k) < 1:
                    log.error("invalid length for dimension key {0}: {1} -> {2}".format(k, name, dimensions))
                    raise InvalidDimensionKey
                if restricted_dimension_chars.search(k) or re.match('^_', k):
                    log.error("invalid characters in dimension key {0}: {1} -> {2}".format(k, name, dimensions))
                    raise InvalidDimensionKey

                if not isinstance(v, (str, unicode)):
                    log.error("invalid dimension value {0} for key {1} must be a string: {2} -> {3}".format(v, k, name,
                                                                                                            dimensions))
                    raise InvalidDimensionValue
                if len(v) > 255 or len(v) < 1:
                    log.error("invalid length dimension value {0} for key {1}: {2} -> {3}".format(v, k, name,
                                                                                                  dimensions))
                    raise InvalidDimensionValue
                if restricted_dimension_chars.search(v):
                    log.error("invalid characters in dimension value {0} for key {1}: {2} -> {3}".format(v, k, name,
                                                                                                         dimensions))
                    raise InvalidDimensionValue

        if not isinstance(name, (str, unicode)):
            log.error("invalid metric name must be a string: {0} -> {1}".format(name, dimensions))
            raise InvalidMetricName
        if len(name) > 255 or len(name) < 1:
            log.error("invalid length for metric name: {0} -> {1}".format(name, dimensions))
            raise InvalidMetricName
        if restricted_name_chars.search(name):
            log.error("invalid characters in metric name: {0} -> {1}".format(name, dimensions))
            raise InvalidMetricName

        if not isinstance(value, (int, long, float)):
            log.error("invalid value {0} is not of number type for metric {1}".format(value, name))
            raise InvalidValue

        if value_meta:
            if not self._valid_value_meta(value_meta, name, dimensions):
                raise InvalidValueMeta

        hostname_to_post = self.get_hostname_to_post(hostname)

        tenant_to_post = delegated_tenant or self.global_delegated_tenant

        dimensions_copy = dimensions.copy()

        if 'hostname' not in dimensions_copy and hostname_to_post:
            dimensions_copy.update({'hostname': hostname_to_post})

        # TODO(joe): Shouldn't device_name be added to dimensions in the check
        #            plugin?  Why is it special cased through so many layers?
        if device_name:
            dimensions_copy.update({'device': device_name})

        # TODO(joe): Decide if hostname_to_post and device_name are necessary
        #            for the context tuple
        context = (name, tuple(dimensions_copy.items()), tenant_to_post,
                   hostname_to_post, device_name)

        if context not in self.metrics:
            self.metrics[context] = metric_class(name,
                                                 dimensions_copy,
                                                 tenant=tenant_to_post)
        cur_time = time()
        if timestamp is not None:
            if cur_time - int(timestamp) > self.recent_point_threshold:
                log.debug("Discarding {0} - ts = {1}, current ts = {2} ".format(name, timestamp, cur_time))
                self.num_discarded_old_points += 1
                return
        else:
            timestamp = cur_time
        self.metrics[context].value_meta = value_meta
        self.metrics[context].sample(value, sample_rate, timestamp)


def get_value_meta_overage(value_meta):
    if len(json.dumps(value_meta)) > VALUE_META_VALUE_MAX_LENGTH:
        return len(json.dumps(value_meta)) - VALUE_META_VALUE_MAX_LENGTH
    return 0



# (C) Copyright 2015,2016 Hewlett Packard Enterprise Development LP

import ast
import logging
import select
import socket

import monasca_agent.common.metrics as metrics_pkg

log = logging.getLogger(__name__)


UDP_SOCKET_TIMEOUT = 5

metric_class = {
    'g': metrics_pkg.Gauge,
    'c': metrics_pkg.Counter,
    'r': metrics_pkg.Rate,
}


class Server(object):
    """A statsd udp server."""

    def __init__(self, aggregator, host, port, forward_to_host=None, forward_to_port=None):
        self.host = host
        self.port = int(port)
        self.address = (self.host, self.port)
        self.aggregator = aggregator
        self.buffer_size = 1024 * 8

        self.running = False

        self.should_forward = forward_to_host is not None

        self.forward_udp_sock = None
        # In case we want to forward every packet received to another statsd server
        if self.should_forward:
            if forward_to_port is None:
                forward_to_port = 8125

            log.info(
                "External statsd forwarding enabled. All packets received will be forwarded to %s:%s" %
                (forward_to_host, forward_to_port))
            try:
                self.forward_udp_sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                self.forward_udp_sock.connect((forward_to_host, forward_to_port))
            except Exception:
                log.exception("Error while setting up connection to external statsd server")

    @staticmethod
    def _parse_metric_packet(packet):
        name_and_metadata = packet.split(':', 1)

        if len(name_and_metadata) != 2:
            raise Exception('Unparseable metric packet: %s' % packet)

        name = name_and_metadata[0]
        metadata = name_and_metadata[1].split('|')

        if len(metadata) < 2:
            raise Exception('Unparseable metric packet: %s' % packet)

        # Submit the metric
        raw_value = metadata[0]
        metric_type = metadata[1]

        if metric_type == 's':
            value = raw_value
        else:
            # Try to cast as an int first to avoid precision issues, then as a
            # float.
            try:
                value = int(raw_value)
            except ValueError:
                try:
                    value = float(raw_value)
                except ValueError:
                    # Otherwise, raise an error saying it must be a number
                    raise Exception('Metric value must be a number: %s, %s' % (name, raw_value))

        # Parse the optional values - sample rate & dimensions.
        sample_rate = 1
        dimensions = {}
        for m in metadata[2:]:
            # Parse the sample rate
            if m[0] == '@':
                sample_rate = float(m[1:])
                assert 0 <= sample_rate <= 1
            # Parse dimensions
            elif m[0] == '#':
                dimensions = ast.literal_eval(m[1:])

        return name, value, metric_type, dimensions, sample_rate

    def submit_packets(self, packets):
        for packet in packets.split("\n"):

            if not packet.strip():
                continue

            if packet.startswith('_e'):
                # Monasca api doesnt support events
                continue

            # todo it seems like this count should be done in the submit_metric method
            self.aggregator.count += 1
            name, value, mtype, dimensions, sample_rate = self._parse_metric_packet(packet)

            if mtype not in metric_class:
                log.warn("metric type {} not supported.".format(mtype))
                continue

            self.aggregator.submit_metric(name,
                                          value,
                                          metric_class[mtype],
                                          dimensions=dimensions,
                                          sample_rate=sample_rate)

    def start(self):
        """Run the server."""
        # Bind to the UDP socket.
        # IPv4 only
        open_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        open_socket.setblocking(0)
        try:
            open_socket.bind(self.address)
        except socket.gaierror:
            if self.address[0] == 'localhost':
                log.warning(
                    "Warning localhost seems undefined in your host file, using 127.0.0.1 instead")
                self.address = ('127.0.0.1', self.address[1])
                open_socket.bind(self.address)

        log.info('Listening on host & port: %s' % str(self.address))

        # Inline variables for quick look-up.
        buffer_size = self.buffer_size
        sock = [open_socket]
        socket_recv = open_socket.recv
        select_select = select.select
        select_error = select.error
        timeout = UDP_SOCKET_TIMEOUT
        should_forward = self.should_forward
        forward_udp_sock = self.forward_udp_sock

        # Run our select loop.
        self.running = True
        while self.running:
            try:
                ready = select_select(sock, [], [], timeout)
                if ready[0]:
                    message = socket_recv(buffer_size)
                    self.submit_packets(message)

                    if should_forward:
                        forward_udp_sock.send(message)
            except select_error as se:
                # Ignore interrupted system calls from sigterm.
                errno = se[0]
                if errno != 4:
                    raise
            except (KeyboardInterrupt, SystemExit):
                break
            except Exception:
                log.exception('Error receiving datagram')

    def stop(self):
        self.running = False



# (C) Copyright 2016 Hewlett Packard Enterprise Development LP


from copy import deepcopy
import errno
import fcntl
import json
import os
import time

from monasca_agent.collector import checks


OK = 0
FAIL = 1

# name used for metrics reported directly by this module
PLUGIN_METRIC_NAME = 'monasca.json_plugin.status'

# Assumes metrics file written every 60 seconds
DEFAULT_STALE_AGE = 60 * 4                # These are too old to report

# Valid attributes of a metric
METRIC_KEYS = ['name', 'metric', 'timestamp', 'value', 'dimensions',
               'value_meta']


def _now():
    """Makes unit testing easier"""
    return time.time()


class JsonPlugin(checks.AgentCheck):
    """Read measurements from JSON-formatted files

    This plugin reads measurements from JSON-formatted files.

    The format of the file is shown in the following example:

    {
        "stale_age": 300,
        "replace_timestamps": false,
        "measurements: [
            {
                "metric": "a_metric",
                "dimensions: ["dim1": "val1"],
                "value: 30.0,
                "timestamp": 1474644040
            },
            {
                "metric": "second_metric",
                "dimensions: ["dim2": "val2"],
                "value: 22.4,
                "timestamp": 1474644040
            },
        ]
    }

    In effect, the file contains a header and a list of measurements. The
    header has the following fields:

    stale_age:

        A time in seconds. If the timestamp of a measurement is
        older than this, this plugin reports a json_plugin.check metric
        with a value of 1. The value_meta contains the name of
        the JSON file that is aged.

        This header is optional. It defaults to 4 minutes

    replace_timestamps:

        A boolean. If set, the next time the plugin is called, it will
        send all the measurements with a timestamp equal to the current
        time (ignoring the timestamp in the measurements list). The
        stale_age value is ignored with this setting.

        This header is optional. It defaults to false.

    measurements:

        This is a list of the measurements, formatted in the same way
        that measurements are presented to the Monasca API. However,
        if replace_timestamps is set, the timestamp key can be omitted
        (since it is set to current time).

    An alternate format for the file is that the header is omitted, i.e.,
    if the first item in the file is a list, it is assumed this is the
    measurement list and the header values are defaulted.
    """

    def __init__(self, name, init_config, agent_config, instances=None,
                 logger=None):
        super(JsonPlugin, self).__init__(name, init_config, agent_config,
                                         instances)
        self.log = logger or self.log
        self.plugin_failures = {}
        self.now = time.time()
        self.posted_metrics = {}

    def _plugin_failed(self, file_name, msg):
        self.plugin_failures[file_name] = msg
        self.log.warn('%s: %s' % (file_name, msg))

    def _plugin_check_metric(self):
        """Generate metric to report status of the plugin"""
        plugin_metric = dict(metric=PLUGIN_METRIC_NAME,
                             dimensions={},
                             value=OK,
                             timestamp=self.now)
        # If there were any failures, put the path
        # and error message into value_meta.
        errors = []
        for path, err in self.plugin_failures.items():
            if err:
                errors.append('%s: %s' % (path, err))
        msg = ''
        if errors:
                msg = ', '.join(errors)
        if msg:
            if len(msg) > 1024:  # keep well below length limit
                msg = msg[:-1021] + '...'
            plugin_metric.update(dict(value_meta=dict(msg=msg),
                                      value=FAIL))
        return plugin_metric

    @staticmethod
    def _take_shared_lock(fd):
        """Take shared lock on a file descriptor

        Assuming the writer of the JSON file also takes a lock, this
        function locks a file descriptor so that we can read the file
        without worrying that the content is changing as we read.

        Raises IOError if lock cannot be taken after a number of attempts.

        :param fd: the file descriptor of the file being read
        """
        max_retries = 5
        delay = 0.02
        attempts = 0
        while True:
            attempts += 1
            try:
                fcntl.flock(fd, fcntl.LOCK_SH | fcntl.LOCK_NB)
                break
            except IOError as err:
                if (err.errno not in [errno.EWOULDBLOCK, errno.EACCES] or
                        attempts > max_retries):
                    raise
                time.sleep(delay * attempts)

    def _load_measurements_from_file(self, file_name):
        handling = {}
        file_data = {'measurements': []}
        try:
            with open(file_name, 'r') as f:
                self._take_shared_lock(f)
                f.seek(0)
                file_data = json.load(f)
        except (ValueError, TypeError) as e:
            self._plugin_failed(file_name,
                                'failed parsing json: %s' % e)
        except Exception as e:  # noqa
            self._plugin_failed(file_name,
                                'loading error: %s' % e)
        try:
            if isinstance(file_data, list):
                metrics = file_data
                handling['stale_age'] = DEFAULT_STALE_AGE
                handling['replace_timestamps'] = False
            else:
                metrics = file_data.get('measurements', [])
                handling['stale_age'] = file_data.get('stale_age',
                                                      DEFAULT_STALE_AGE)
                handling['replace_timestamps'] = file_data.get(
                    'replace_timestamps', False)
        except Exception as e:  # noqa
            self._plugin_failed(file_name,
                                'unable to process file contents: %s' % e)
            metrics = []

        metrics = self._filter_metrics(metrics, file_name)
        return self._remove_duplicate_metrics(handling, metrics, file_name)

    def _filter_metrics(self, metrics, file_name):
        """Remove invalid metrics from the metric list

        Validate and clean up so the metric is suitable for passing to
        AgentCheck.gauge(). The metric might be invalid (e.g., value_meta too
        long), but that's not our concern here.
        """
        invalid_metric = None
        valid_metrics = []
        for metric in metrics:
            if not isinstance(metric, dict):
                invalid_metric = metric  # not a dict
                continue
            for key in metric.keys():
                if key not in METRIC_KEYS:
                    invalid_metric = metric  # spurious attribute
                    continue
            if 'name' not in metric.keys() and 'metric' not in metric.keys():
                invalid_metric = metric  # missing name
                continue
            if 'value' not in metric.keys():
                invalid_metric = metric  # missing value
                continue

            if 'name' in metric:
                # API use 'name'; AgentCheck uses 'metric'
                metric['metric'] = metric.get('name')
                del metric['name']
            if not metric.get('dimensions', None):
                metric['dimensions'] = {}
            valid_metrics.append(metric)

        if invalid_metric:
            # Only report one invalid metric per file
            self._plugin_failed(file_name, 'invalid metric found: %s' % metric)
        return valid_metrics

    def _remove_duplicate_metrics(self, handling, metrics, file_name):
        """Remove metrics if we've already reported them

        We track the metrics we've posted to the Monasca Agent This allows us
        to discard duplicate metrics. The most common cause of duplicates is
        that the agent runs more often than the update period of the JSON file.

        We also discard metrics that seem stale. This can occur when the
        program creating the metrics file has died, so the JSON file
        does not update with new metrics.

        :param: handling: options for how measurements are handled
        :param metrics: The metrics we found in the JSON file
        :param file_name: the path of the JSON file
        :returns: A list of metrics that should be posted
        """

        # Set timestamp if asked
        if handling['replace_timestamps']:
            for metric in metrics:
                metric['timestamp'] = self.now
            # Since we've set the timestamp, these are unique (not duplicate)
            # so no further processing is required
            return metrics

        # Remove metrics we've already posted. Also remove stale metrics.
        if file_name not in self.posted_metrics:
            self.posted_metrics[file_name] = []
        stale_metrics = False
        for metric in deepcopy(metrics):
            if ((self.now - metric.get('timestamp', 0)) >
                    handling.get('stale_age')):
                metrics.remove(metric)  # too old
                stale_metrics = True
            elif metric in self.posted_metrics[file_name]:
                metrics.remove(metric)  # already sent to Monasca
            else:
                # New metric; not stale.
                self.posted_metrics[file_name].append(metric)

        # Purge really old metrics from posted
        for metric in list(self.posted_metrics[file_name]):
            if ((self.now - metric.get('timestamp', 0)) >=
                    handling.get('stale_age') * 2):
                self.posted_metrics[file_name].remove(metric)

        if stale_metrics:
            self._plugin_failed(file_name, 'Metrics are older than %s seconds;'
                                           ' file not updating?' %
                                handling.get('stale_age'))
        return metrics

    def _get_metrics(self):
        reported = []
        for file_name in self.metrics_files:
            metrics = self._load_measurements_from_file(file_name)
            for metric in metrics:
                reported.append(metric)
        return reported

    def _load_instance_config(self, instance):
        self.metrics_files = []
        self.metrics_dir = instance.get('metrics_dir', '')
        if self.metrics_dir:
            self.plugin_failures[self.metrics_dir] = ''
            try:
                file_names = os.listdir(self.metrics_dir)
                for name in [os.path.join(self.metrics_dir, name) for
                             name in file_names]:
                    # .json extension protects from reading .swp and similar
                    if os.path.isfile(name) and name.lower().endswith('.json'):
                        self.metrics_files.append(name)
            except OSError as err:
                self._plugin_failed(self.metrics_dir,
                                    'Error processing: %s' % err)
        else:
            metric_file = instance.get('metrics_file', '')
            if metric_file:
                self.metrics_files = [metric_file]
        self.log.debug('Using metrics files %s' % ','.join(self.metrics_files))
        for file_name in self.metrics_files:
            self.plugin_failures[file_name] = ''

    def check(self, instance):
        self._load_instance_config(instance)
        all_metrics = []
        self.now = _now()

        # Load measurements from files
        metrics = self._get_metrics()
        all_metrics.extend(metrics)

        # Add this plugin status
        all_metrics.append(self._plugin_check_metric())

        for metric in all_metrics:
            # apply any instance dimensions that may be configured,
            # overriding any dimension with same key that check has set.
            metric['dimensions'] = self._set_dimensions(metric['dimensions'],
                                                        instance)
            self.log.debug('Posting metric: %s' % metric)
            try:
                self.gauge(**metric)
            except Exception as e:  # noqa
                self.log.exception('Exception while reporting metric: %s' % e)



import time
import unittest
import logging
logger = logging.getLogger()
from monasca_agent.common.exceptions import UnknownValue, CheckException, Infinity
from monasca_agent.collector.checks.check import Check
from monasca_agent.common.aggregator import MetricsAggregator


class TestCore(unittest.TestCase):

    "Tests to validate the core check logic"

    def setUp(self):
        self.c = Check(logger)
        self.c.gauge("test-metric")
        self.c.counter("test-counter")

    def test_gauge(self):
        self.assertEqual(self.c.is_gauge("test-metric"), True)
        self.assertEqual(self.c.is_counter("test-metric"), False)
        self.c.save_sample("test-metric", 1.0)
        # call twice in a row, should be invariant
        self.assertEqual(self.c.get_sample("test-metric"), 1.0)
        self.assertEqual(self.c.get_sample("test-metric"), 1.0)
        self.assertEqual(self.c.get_sample_with_timestamp("test-metric")[1], 1.0)
        # new value, old one should be gone
        self.c.save_sample("test-metric", 2.0)
        self.assertEqual(self.c.get_sample("test-metric"), 2.0)
        self.assertEqual(len(self.c._sample_store["test-metric"]), 1)
        # with explicit timestamp
        self.c.save_sample("test-metric", 3.0, 1298066183.607717)
        self.assertEqual(self.c.get_sample_with_timestamp(
            "test-metric"), (1298066183.607717, 3.0, None, None))
        # get_samples()
        self.assertEqual(self.c.get_samples(), {"test-metric": 3.0})

    def testEdgeCases(self):
        self.assertRaises(UnknownValue, self.c.get_sample, "unknown-metric")
        # same value
        self.c.save_sample("test-counter", 1.0, 1.0)
        self.c.save_sample("test-counter", 1.0, 1.0)
        self.assertRaises(Infinity, self.c.get_sample, "test-counter")

    def test_counter(self):
        self.c.save_sample("test-counter", 1.0, 1.0)
        self.assertRaises(UnknownValue, self.c.get_sample, "test-counter", expire=False)
        self.c.save_sample("test-counter", 2.0, 2.0)
        self.assertEqual(self.c.get_sample("test-counter", expire=False), 1.0)
        self.assertEqual(self.c.get_sample_with_timestamp(
            "test-counter", expire=False), (2.0, 1.0, None, None))
        self.assertEqual(self.c.get_samples(expire=False), {"test-counter": 1.0})
        self.c.save_sample("test-counter", -2.0, 3.0)
        self.assertRaises(UnknownValue, self.c.get_sample_with_timestamp, "test-counter")

    def test_dimensions(self):
        # Test metric dimensions
        now = int(time.time())
        # dimensions metrics
        self.c.save_sample(
            "test-counter", 1.0, 1.0, dimensions={"dim1": "value1", "dim2": "value2"})
        self.c.save_sample(
            "test-counter", 2.0, 2.0, dimensions={"dim1": "value1", "dim2": "value2"})
        # Only 1 point recording for this combination of dimensions, won't be sent
        self.c.save_sample(
            "test-counter", 3.0, 3.0, dimensions={"dim1": "value1", "dim3": "value3"})
        self.c.save_sample("test-metric", 3.0, now, dimensions={"dim3": "value3", "dim4": "value4"})
        # Arg checks
        self.assertRaises(
            CheckException, self.c.save_sample, "test-metric", 4.0, now + 5, dimensions="abc")
        # This is a different combination of dimensions
        self.c.save_sample("test-metric", 3.0, now, dimensions={"dim5": "value5", "dim3": "value3"})
        results = sorted(self.c.get_metrics())
        self.assertEqual(results,
                         [("test-counter", 2.0, 1.0, {"dimensions": {"dim1": "value1", "dim2": "value2"}}),
                          ("test-metric", now, 3.0,
                           {"dimensions": {"dim3": "value3", "dim4": "value4"}}),
                          ("test-metric", now, 3.0,
                           {"dimensions": {"dim3": "value3", "dim5": "value5"}})
                          ])
        # dimensions metrics are not available through get_samples anymore
        self.assertEqual(self.c.get_samples(), {})

    def test_samples(self):
        self.assertEqual(self.c.get_samples(), {})
        self.c.save_sample("test-metric", 1.0, 0.0)  # value, ts
        self.c.save_sample("test-counter", 1.0, 1.0)  # value, ts
        self.c.save_sample("test-counter", 4.0, 2.0)  # value, ts
        assert "test-metric" in self.c.get_samples_with_timestamps(
            expire=False), self.c.get_samples_with_timestamps(expire=False)
        self.assertEqual(self.c.get_samples_with_timestamps(
            expire=False)["test-metric"], (0.0, 1.0, None, None))
        assert "test-counter" in self.c.get_samples_with_timestamps(
            expire=False), self.c.get_samples_with_timestamps(expire=False)
        self.assertEqual(self.c.get_samples_with_timestamps(
            expire=False)["test-counter"], (2.0, 3.0, None, None))

    def test_name(self):
        self.assertEqual(self.c.normalize("metric"), "metric")
        self.assertEqual(self.c.normalize("metric", "prefix"), "prefix.metric")
        self.assertEqual(self.c.normalize("__metric__", "prefix"), "prefix.metric")
        self.assertEqual(
            self.c.normalize("abc.metric(a+b+c{}/5)", "prefix"), "prefix.abc.metric_a_b_c_5")
        self.assertEqual(
            self.c.normalize(
                "VBE.default(127.0.0.1,,8080).happy",
                "varnish"),
            "varnish.VBE.default_127.0.0.1_8080.happy")


class TestAggregator(unittest.TestCase):

    def setUp(self):
        self.aggr = MetricsAggregator('test-aggr')

    def test_dupe_tags(self):
        self.aggr.increment('test-counter', 1, dimensions={'a': 'avalue', 'b': 'bvalue'})
        self.aggr.increment(
            'test-counter', 1, dimensions={'a': 'avalue', 'b': 'bvalue', 'b': 'bvalue'})
        self.assertEqual(len(self.aggr.metrics), 1, self.aggr.metrics)
        metric = self.aggr.metrics.values()[0]
        self.assertEqual(metric.value, 2)

if __name__ == '__main__':
    unittest.main()



# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP

""" Util functions to assist in detection.
"""
import logging
import subprocess
from subprocess import CalledProcessError
from subprocess import PIPE
from subprocess import Popen

from monasca_agent.common.psutil_wrapper import psutil
from monasca_setup import agent_config

log = logging.getLogger(__name__)


# check_output was introduced in python 2.7, function added
# to accommodate python 2.6
try:
    check_output = subprocess.check_output
except AttributeError:
    def check_output(*popenargs, **kwargs):
        if 'stdout' in kwargs:
            raise ValueError('stdout argument not allowed, it will be overridden.')
        process = Popen(stdout=PIPE, *popenargs, **kwargs)
        output, unused_err = process.communicate()
        retcode = process.poll()
        if retcode:
            cmd = kwargs.get("args")
            if cmd is None:
                cmd = popenargs[0]
            raise CalledProcessError(retcode, cmd)
        return output


def find_process_cmdline(search_string):
    """Simple function to search running process for one with cmdline
    containing search_string.
    """
    for process in psutil.process_iter():
        try:
            process_cmdline = ' '.join(process.as_dict()['cmdline'])
            if (search_string in process_cmdline and
               'monasca-setup' not in process_cmdline):
                return process
        except psutil.NoSuchProcess:
            continue

    return None


def find_process_name(pname):
    """Simple function to search running process for one with pname.
    """
    for process in psutil.process_iter():
        try:
            if pname == process.as_dict()['name']:
                return process
        except psutil.NoSuchProcess:
            continue

    return None


def find_process_service(sname):
    """Simple function to call systemctl (service) to check if a service is running.
    """
    try:
        subprocess.check_call(['service', sname, 'status'], stdout=PIPE, stderr=PIPE)
        return True
    except subprocess.CalledProcessError:
        return False

    return False


def find_addrs_listening_on_port(port, kind='inet'):
    """Return the list of IP addresses which are listening
    on the specified port.
    """
    listening = []
    for connection in psutil.net_connections(kind):
        if (connection.status == psutil.CONN_LISTEN and
                connection.laddr[1] == int(port)):
            listening.append(connection.laddr[0])
    return listening


def find_addr_listening_on_port_over_tcp(port):
    """Return the IP address which is listening on the specified TCP port."""
    ip = find_addrs_listening_on_port(port, 'tcp')
    if ip:
        return ip[0].lstrip("::ffff:")


def watch_process(search_strings, service=None, component=None,
                  exact_match=True, detailed=True, process_name=None):
    """Takes a list of process search strings and returns a Plugins object with the config set.
        This was built as a helper as many plugins setup process watching
    """
    config = agent_config.Plugins()

    # Fallback to default process_name strategy if process_name is not defined
    process_name = process_name if process_name else search_strings[0]
    parameters = {'name': process_name,
                  'detailed': detailed,
                  'exact_match': exact_match,
                  'search_string': search_strings}

    dimensions = _get_dimensions(service, component)
    if len(dimensions) > 0:
        parameters['dimensions'] = dimensions

    config['process'] = {'init_config': None,
                         'instances': [parameters]}
    return config


def watch_process_by_username(username, process_name, service=None, component=None, detailed=True):
    """Takes a user and returns a Plugins object with the config set for a process check by user.
        This was built as a helper as many plugins setup process watching.
    """
    config = agent_config.Plugins()

    parameters = {'name': process_name,
                  'detailed': detailed,
                  'username': username}

    dimensions = _get_dimensions(service, component)
    if len(dimensions) > 0:
        parameters['dimensions'] = dimensions

    config['process'] = {'init_config': None,
                         'instances': [parameters]}
    return config


def watch_file_size(directory_name, file_names, file_recursive=False,
                    service=None, component=None):
    """Takes a directory, a list of files, recursive flag and returns a
        Plugins object with the config set.
    """
    config = agent_config.Plugins()
    parameters = {'directory_name': directory_name,
                  'file_names': file_names,
                  'recursive': file_recursive}

    dimensions = _get_dimensions(service, component)
    if len(dimensions) > 0:
        parameters['dimensions'] = dimensions

    config['file_size'] = {'init_config': None,
                           'instances': [parameters]}
    return config


def watch_directory(directory_name, service=None, component=None):
    """Takes a directory name and returns a Plugins object with the config set.
    """
    config = agent_config.Plugins()
    parameters = {'directory': directory_name}

    dimensions = _get_dimensions(service, component)
    if len(dimensions) > 0:
        parameters['dimensions'] = dimensions

    config['directory'] = {'init_config': None,
                           'instances': [parameters]}
    return config


def service_api_check(name, url, pattern,
                      use_keystone=True, service=None, component=None):
    """Setup a service api to be watched by the http_check plugin.
    """
    config = agent_config.Plugins()
    parameters = {'name': name,
                  'url': url,
                  'match_pattern': pattern,
                  'timeout': 10,
                  'use_keystone': use_keystone}

    dimensions = _get_dimensions(service, component)
    if len(dimensions) > 0:
        parameters['dimensions'] = dimensions

    config['http_check'] = {'init_config': None,
                            'instances': [parameters]}
    return config


def _get_dimensions(service, component):
    dimensions = {}
    # If service parameter is set in the plugin config, add the service dimension which
    # will override the service in the agent config
    if service:
        dimensions.update({'service': service})

    # If component parameter is set in the plugin config, add the component dimension which
    # will override the component in the agent config
    if component:
        dimensions.update({'component': component})

    return dimensions



# (c) Copyright 2015-2016 Hewlett Packard Enterprise Development LP

import grp
import logging
import os
import yaml

import monasca_setup.agent_config
import monasca_setup.detection

log = logging.getLogger(__name__)


class Postfix(monasca_setup.detection.Plugin):
    """If postfix is running install the default config.
    """
    def _detect(self):
        """Run detection, set self.available True if the service is detected.

        """
        # Detect Agent's OS username by getting the group owner of confg file
        try:
            gid = os.stat('/etc/monasca/agent/agent.yaml').st_gid
            agent_user = grp.getgrgid(gid)[0]
        except OSError:
            agent_user = None
        if monasca_setup.detection.find_process_cmdline('postfix') is not None:
            # Test for sudo access
            test_sudo = os.system('sudo -l -U {0} find /var/spool/postfix/incoming -type f > /dev/null'.format(agent_user))
            if test_sudo != 0:
                log.info("Postfix found but the required sudo access is not configured.\n\t" +
                         "Refer to plugin documentation for more detail")
                return False
            self.available = True
        else:
            self.available = False

    def build_config(self):
        """Build the config as a Plugins object and return.

        """
        # A bit silly to parse the yaml only for it to be converted back but this
        # plugin is the exception not the rule
        with open(os.path.join(self.template_dir, 'conf.d/postfix.yaml.example'), 'r') as postfix_template:
            default_net_config = yaml.safe_load(postfix_template.read())
        config = monasca_setup.agent_config.Plugins()
        config['postfix'] = default_net_config
        return config

    def dependencies_installed(self):
        return True



# (C) Copyright 2016-2017 Hewlett Packard Enterprise Development LP

import ConfigParser
import logging

from monasca_agent.common.psutil_wrapper import psutil
import monasca_setup.agent_config
from monasca_setup.detection import Plugin
from monasca_setup.detection.utils import find_process_name


log = logging.getLogger(__name__)


class VCenter(Plugin):
    """Configures ESX Cluster monitoring through VCenter"""

    def _detect(self):
        """Method to detect the nova-compute service,
        if found set the nova.conf, the flags under vmware section will be used to
        configure the plugin, else the args are used to configure.
        """
        # Find the nova compute process and locate its conf
        process_exist = find_process_name('nova-compute') is not None
        # for cases where this plugin and nova-compute service runs separately
        # we will configure the plugin with given args.
        # so, we have to set these below variables
        self.nova_conf = self.get_nova_config_file() if process_exist else None
        has_config_file_or_args = (self.nova_conf is not None or
                                   self.args is not None)
        self.available = process_exist and has_config_file_or_args
        if not self.available:
            if not process_exist:
                log.error('Nova-compute process does not exist.')
            elif not has_config_file_or_args:
                log.error(('Nova-compute process exists but '
                           'the configuration file was not detected and no '
                           'arguments were given.'))

    def get_nova_config_file(self):
        nova_conf = None
        for proc in psutil.process_iter():
            try:
                cmd = proc.as_dict(['cmdline'])['cmdline']
                if len(cmd) > 2 and 'python' in cmd[0] and 'nova-compute' in cmd[1]:
                    params = [cmd.index(y) for y in cmd if 'hypervisor.conf' in y]
                    if not params:
                        # The configuration file is not found, skip
                        continue
                    else:
                        param = params[0]
                    if '=' in cmd[param]:
                        nova_conf = cmd[param].split('=')[1]
                    else:
                        nova_conf = cmd[param]
            except IOError:
                # Process has already terminated, ignore
                continue
        return nova_conf

    def build_config(self):
        """Build the config as a Plugins object and return back.
        """
        config = monasca_setup.agent_config.Plugins()

        if self.dependencies_installed():
            nova_cfg = ConfigParser.SafeConfigParser()
            instance = {}
            if self.nova_conf is None:
                log.warn("Nova compute configuration file was not found.")
                if self.args:
                    # read from arg list
                    instance = self._read_from_args(instance)
                else:
                    # get the default config format
                    instance = self._config_format()
            else:
                log.info("Using nova configuration file {0}".format(self.nova_conf))
                nova_cfg.read(self.nova_conf)
                cfg_section = 'vmware'

                # extract the vmware config from nova.conf and build instances
                if (nova_cfg.has_option(cfg_section, 'host_ip')
                        and nova_cfg.has_option(cfg_section, 'host_username')
                        and nova_cfg.has_option(cfg_section, 'host_password')
                        and nova_cfg.has_option(cfg_section, 'host_port')
                        and nova_cfg.has_option(cfg_section, 'cluster_name')):

                    instance = {
                        'vcenter_ip': nova_cfg.get(cfg_section, 'host_ip'),
                        'username': nova_cfg.get(cfg_section, 'host_username'),
                        'password': nova_cfg.get(cfg_section, 'host_password', raw=True),
                        'port': int(nova_cfg.get(cfg_section, 'host_port')),
                        'clusters': [nova_cfg.get(cfg_section, 'cluster_name')]
                    }
                else:
                    log.warn("One or more configuration parameters are missing"
                             " host_ip, host_username, host_password,"
                             " host_port, cluster_name")
                    # put default format
                    instance = self._config_format()
            config['vcenter'] = {'init_config': {},
                                 'instances': [instance]}
        return config

    def _config_format(self):
        """Default configuration format for vcenter plugin
        """
        instance = {'vcenter_ip': None,
                    'username': None,
                    'password': None,
                    'port': None,
                    'clusters': []}
        return instance

    def _read_from_args(self, instance):
        """Read the args and build the instance config
        """
        for arg in self.args:
            if arg == 'clusters':
                cls_lst = self.args[arg].split(',')
                instance[arg] = cls_lst
            else:
                instance[arg] = self.args[arg]
        return instance

    def dependencies_installed(self):
        """Import the dependencies.
        """
        return True
